#include <vtkConeSource.h>
#include <vtkLight.h>
#include <vtkMaterialLibrary.h>
#include <vtkLightCollection.h>
#include <vtkCallbackCommand.h>
#include <vtkProperty.h>
#include <vtkPolyDataMapper.h>
#include <vtkActor.h>
#include <vtkProp.h>
#include <vtkRenderer.h>
#include <vtkRendererCollection.h>
#include <vtkXOpenGLRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkCamera.h>
#include <vtkSmartPointer.h>

#include <unistd.h>
#include <stdio.h>

#define VTK_NEW(type, name) vtkSmartPointer<type> name = vtkSmartPointer<type>::New();

#define PRINT_VECT(title, x) \
    std::cout << title << ": (" << x[0] << ", " << x[1] << ", " << x[2] << ")" << std::endl

void setCamera(vtkCamera *cam)
{
    cam->SetClippingRange(5, -5);
    cam->SetPosition(0, 0, 10);
    cam->SetDistance(10);
    cam->SetFocalPoint(0, 0, 0);
    cam->ComputeViewPlaneNormal();
    cam->SetViewUp(0, 1, 0);
}

void keyHandler(vtkObject *obj, unsigned long eid, void *clientdata, void *calldata)
{
	vtkRenderWindowInteractor *iren = static_cast<vtkRenderWindowInteractor*>(obj);
	vtkXOpenGLRenderWindow *win = static_cast<vtkXOpenGLRenderWindow *>(clientdata);
	vtkCamera *cam = win->GetRenderers()->GetFirstRenderer()->GetActiveCamera();

	int c = iren->GetKeyCode();

	double d;

	switch (c) {
		case 'r':
			cam->Azimuth(5);
			break;

		case 'R':
			cam->Azimuth(-5);
			break;

		case 'z':
			cam->Dolly(1.1);
			break;

		case 'Z':
			cam->Dolly(0.9);
			break;

		case 'i':
			cam->Azimuth(180);
			break;

		case 'p':
			cam->SetParallelProjection(1);
			break;

		case 'P':
			cam->SetParallelProjection(0);

		case '_':
			setCamera(cam);
			break;

		default:
			std::cout << "Got key: " << c << std::endl;
	}

	win->Render();
}

int main(void)
{
    /* visualization */
    VTK_NEW(vtkConeSource, cone);
    cone->SetCenter(0, 0, 0);
    cone->SetRadius(1);
    cone->SetHeight(3);
    cone->SetResolution(5);

    VTK_NEW(vtkPolyDataMapper, mapper);
    mapper->SetInputConnection(cone->GetOutputPort());

    VTK_NEW(vtkActor, actor);
    actor->SetMapper(mapper);
    actor->GetProperty()->SetColor(1.0, 0.4, 0.4);
    actor->GetProperty()->SetAmbient(0.7);
    actor->GetProperty()->LoadMaterial("GLSLTwisted");

    VTK_NEW(vtkRenderer, renderer);
    renderer->AddActor(actor);
    renderer->SetAmbient(0.5, 0.6, 0.7);

    /* print available materials */
    const char **mat = vtkMaterialLibrary::GetListOfMaterialNames();
    unsigned int nof_mat = vtkMaterialLibrary::GetNumberOfMaterials();
    for (int i=0; i<nof_mat; i++) {
		 std::cout << mat[0] << std::endl;
    }

    /* Camera */
    vtkSmartPointer<vtkCamera> cam = renderer->GetActiveCamera();
    setCamera(cam);

    /* Lights */
    VTK_NEW(vtkLight, light);
    light->SetAmbientColor(0.8, 0.5, 0.3);
    light->SetFocalPoint(0, 0, 0);
    light->SetPosition(4, 1, 0);
    renderer->AddLight(light);

    VTK_NEW(vtkXOpenGLRenderWindow, win);
    VTK_NEW(vtkRenderWindowInteractor, iren);
    win->AddRenderer(renderer);
    iren->SetRenderWindow(win);

	 /* Callback */
	 VTK_NEW(vtkCallbackCommand, keypressCB);
	 keypressCB->SetCallback(keyHandler);
	 keypressCB->SetClientData(win);
	 iren->AddObserver(vtkCommand::KeyPressEvent, keypressCB);

    iren->Initialize();
    win->Render();
    iren->Start();

    return (0);
}
